import { browser } from '$app/environment';
import { onMount } from 'svelte';
import { writable } from 'svelte/store';

export type ThemeModeType = 'light' | 'dark';

export const themeMode = writable<ThemeModeType>('light', (set) => {
	if (!browser) return;

	set((document.documentElement.getAttribute('theme-base') as ThemeModeType) || 'light');
});

/**
 * Send a theme to the server so that it can be saved as a cookie.
 * @param theme - theme to send to server
 */
export const forwardThemeToServer = async (theme: ThemeModeType) =>
	fetch(`/?/setTheme&themeMode=${theme}`, {
		method: 'POST',
		body: new FormData(),
		credentials: 'include',
		headers: {
			'x-sveltekit-action': 'true'
		}
	});

export const forwardInitialTheme = () =>
	onMount(() => {
		// If a cookie-set theme is not present, set the theme to current user agent theme.
		const hasServerTheme = !!document.documentElement.getAttribute('theme-base');
		const uaTheme = window?.matchMedia?.('(prefers-color-scheme: dark)');
		const uaMode: ThemeModeType = uaTheme?.matches ? 'dark' : 'light';
		const uaSetter = () => {
			themeMode.set(uaTheme?.matches ? 'dark' : 'light');
		};

		if (!hasServerTheme) {
			themeMode.set(uaMode);
		}

		const unsubscribeForward = themeMode.subscribe(async (theme) => {
			document.documentElement.setAttribute('theme-base', theme);
			await forwardThemeToServer(theme);
		});

		uaTheme?.addEventListener?.('change', uaSetter);
		return () => {
			unsubscribeForward();
			uaTheme?.removeEventListener('change', uaSetter);
		};
	});
